package org.jamon.node;

import java.util.ArrayList;
import java.util.List;

import org.jamon.api.Location;
public class AbstractComponentCallNode extends AbstractCallNode
{
  public AbstractComponentCallNode(Location p_location, AbstractPathNode p_callPath, AbstractParamsNode p_params)
  {
    super(p_location);
    if ((m_callPath = p_callPath) == null)
      { throw new NullPointerException(); }
    if ((m_params = p_params) == null)
      { throw new NullPointerException(); }
  }

  @Override public void apply(Analysis p_analysis)
  {
    p_analysis.caseAbstractComponentCallNode(this);
  }

  private final AbstractPathNode m_callPath;
  public AbstractPathNode getCallPath() { return m_callPath; }

  private final List<GenericCallParam> m_genericParams = new ArrayList<GenericCallParam>();
  public AbstractComponentCallNode addGenericParam(GenericCallParam p_genericParam)
  {
    if (p_genericParam == null) { throw new NullPointerException(); }
    m_genericParams.add(p_genericParam);
    return this;
  }
  public List<GenericCallParam> getGenericParams() { return m_genericParams; }

  private final AbstractParamsNode m_params;
  public AbstractParamsNode getParams() { return m_params; }

  @Override public boolean equals(Object p_obj)
  {
    return p_obj != null
        && super.equals(p_obj)
      && m_callPath.equals(((AbstractComponentCallNode) p_obj).m_callPath)
      && m_genericParams.equals(((AbstractComponentCallNode) p_obj).m_genericParams)
      && m_params.equals(((AbstractComponentCallNode) p_obj).m_params)
    ;
  }

  @Override public int hashCode()
  {
    return super.hashCode()
      ^ m_callPath.hashCode()
      ^ m_genericParams.hashCode()
      ^ m_params.hashCode()
    ;
  }

  @Override protected void propertiesToString(StringBuilder p_buffer)
  {
    super.propertiesToString(p_buffer);
    addProperty(p_buffer, "callPath", m_callPath);
    addPropertyList(p_buffer, "genericParam", m_genericParams);
    addProperty(p_buffer, "params", m_params);
  }

}
